/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.gui;

import java.awt.Component;
import java.awt.Graphics;

import javax.swing.Icon;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;

import ch.cern.lhcb.xmleditor.specific.IconsManager;
import ch.cern.lhcb.xmleditor.specific.XmlDisplay;

/**
 * this is an improvement of DefaultTreeCellRenderer allowing to change the
 * displayed icon for a given node, depending of its name. 
 * 
 * @author Sebastien Ponce
 */
public class TreeCellRenderer extends DefaultTreeCellRenderer {

    /**
     * an icon manager to be used by this renderer
     */
    private IconsManager iconsManager;

    /**
     * says whether to underline the current Cell or not
     */
    private boolean underlined;

    /**
     * says whether to overline the current Cell or not
     */
    private boolean overlined;

    /**
     * says whether to surround the current Cell or not
     */
    private boolean surrounded;

    /**
     * Default constructor
     * @param iconsManager  icons manager
     */
    public TreeCellRenderer(IconsManager iconsManager) {
        // calls the super constructor
        super();
        // sets the fields
        this.iconsManager = iconsManager;
        this.underlined = false;
        this.overlined = false;
        this.surrounded = false;
    }

    /**
     * overrides the corresponding method of DefaultTReeCellRenderer allows a
     * finer control of the icon displayed for a given node
     */
    public Component getTreeCellRendererComponent(JTree tree, Object value,
            boolean selected, boolean expanded, boolean leaf, int row,
            boolean hasFocus) {
        super.getTreeCellRendererComponent(tree, value, selected, expanded,
                leaf, row, hasFocus);
        // sets the icon
        chooseAndSetIcon(value);
        // by default, no overline, no underline and no surround
        this.underlined = false;
        this.overlined = false;
        this.surrounded = false;
        // computes whether underline, overline or surround is needed
        XmlTree xmlTree = (XmlTree) tree;
        int draggedRow = xmlTree.getDraggedRow();
        if (draggedRow != 1) {
            TreePath treePath = xmlTree.getPathForRow(draggedRow);
            if (treePath != null) {
                if (treePath.getLastPathComponent() == value) {
                    this.underlined = xmlTree.getDraggedRowOffset() == javax.swing.SwingConstants.BOTTOM;
                    this.overlined = xmlTree.getDraggedRowOffset() == javax.swing.SwingConstants.TOP;
                    this.surrounded = xmlTree.getDraggedRowOffset() == javax.swing.SwingConstants.CENTER;
                }
            }
        }
        // returns result
        return this;
    }

    public void paint(Graphics g) {
        // default paint method
        super.paint(g);
        // set the color in harmony with the default cell color
        g.setColor(this.backgroundSelectionColor);
        // if underlined
        if (underlined) {
            g.fillRect(0, this.getHeight() - 2, this.getWidth() - 1, this
                    .getHeight() - 1);
        }
        // if overlined
        if (overlined) {
            g.fillRect(0, 0, this.getWidth() - 1, 1);
        }
        // if surrounded, actually fill the cell
        if (surrounded) {
            g.drawRect(0, 0, this.getWidth() - 1, this.getHeight() - 1);
        }
    }

    /**
     * Chooses an appropriate icon by looking at the node name and sets it
     * 
     * @param value the TreeNode to be processed
     */
    private void chooseAndSetIcon(Object value) {
        // first gets the node and its name
        XmlTreeNode treeNode = (XmlTreeNode) value;
        String name = treeNode.getNode().getNodeName();
        // chooses the icon depending on the name
        if (treeNode.getOldNode() != null && !treeNode.isReference()) {
            name = name + "deref";
        }
        XmlDisplay xmlDisplay = treeNode.getXmlDisplay();
        if (xmlDisplay != null) {
            if (!xmlDisplay.isInDtd(treeNode.getNode().getNodeName())) {
                if (!name.startsWith("#")) {
                    name = "notindtd";
                }
            }
        }
        Icon icon = iconsManager.getIcon(name);
        if (icon != null) {
            setIcon(icon);
        }
    }

}