/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.undo;

import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;

import org.w3c.dom.Document;

import ch.cern.lhcb.xmleditor.xerces.ElementImpl;
import ch.cern.lhcb.xmleditor.xerces.TreeElement;

/**
 * this is an implementation of the abstract class AbstractUndoableEdit that
 * defines the undoable element addition in an XmlTree.
 */

public class UndoableElementAddition extends AbstractUndoableEdit {

    /**
     * the node added
     */
    private TreeElement node;

    /**
     * the parent of the added node
     */
    private ElementImpl parent;

    /**
     * the node following the added node (or null if none)
     */
    private TreeElement neighbour;

    /**
     * Constructor. Specifies the node concerned, its parent and its neighbour.
     * 
     * @param document the document this Edit is associated with
     * @param node the node added
     * @param parent the parent of the added node
     * @param neighbour the node following the node added (or null if none)
     */
    public UndoableElementAddition(Document document, TreeElement node,
            ElementImpl parent, TreeElement neighbour) {
        super(document);
        this.node = node;
        this.parent = parent;
        this.neighbour = neighbour;
    }

    /**
     * Redoes the operation represented by this edit
     * 
     * @throws CannotRedoException if canRedo() returns false
     */
    public void redo() throws CannotRedoException {
        super.redo();
        // adds the new node to its parent
        parent.insertBefore(node, neighbour, true);
    }

    /**
     * Undoes the operation represented by this edit
     * 
     * @throws CannotUndoException if canUndo() returns false
     */
    public void undo() throws CannotUndoException {
        super.undo();
        // removes the node from its parent
        parent.removeChild(node);
    }

    /**
     * returns an appropriate description of the operation this edit represents.
     * 
     * @return a description of the operation
     */
    public String getPresentationName() {
        return node.getNodeName() + " addition";
    }

}