/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview;

import java.util.Vector;

/**
 * Class that extends standard functionality of <code>JDesktopPane</code>
 */
public class DataDesktop extends javax.swing.JDesktopPane implements
		javax.swing.event.InternalFrameListener {

	private WindowMenu windowMenu;

	private boolean createFrame = true;

	/** Holds value of property title. */
	private String default_title = ResourceManager.getResource("Results_View");

	private int frameIndex = 1;

	/** Holds value of property preview. */
	private PrintPreview preview;

	/** Holds value of task progress. */
	private TaskProgress tasks;

	/**
	 * Creates new DataDesktop
	 */
	public DataDesktop() {
		super();
		windowMenu = new WindowMenu(this);
		preview = new PrintPreview(null, this);
		tasks = new TaskProgress();

		DataFrame f = new DataFrame("", this);
	    f.addNotify();
			
		setWindowActions(false);
	}

	/**
	 * Returns the Vector of DataFrames on this desktop
	 * 
	 * @return Vector of DataFrames
	 */
	public Vector getDataFrames() {
		javax.swing.JInternalFrame[] frames = this.getAllFrames();
		Vector data = new Vector();

		for (int i = 0; i < frames.length; i++) {
			if ((frames[i] instanceof DataFrame) && frames[i].isVisible())
				data.addElement(frames[i]);
		}
		return data;
	}

	/**
	 * Returns the active (topmost) frame on the desktop
	 * 
	 * @return active frame
	 */
	public DataFrame getActiveFrame() {
		Vector frames = getDataFrames();
		if ((frames != null) && (frames.size() > 0)) {
			return (DataFrame) frames.get(0);
		} else {
			return null;
		}
	}

	/**
	 * Creates and adds new DataFrame Note: call <code>show()</code> to make
	 * this frame visible
	 * 
	 * @param title
	 *            frame title
	 * @return created DataFrame
	 */
	public DataFrame addFrame(String title) {
		DataFrame f = getActiveFrame();

		if ((title == null) && (title.length() == 0))
			title = default_title;

		if (createFrame || (getDataFrames().size() == 0)) {
			f = new DataFrame(title, this);
			f.setNumber(frameIndex);
			frameIndex++;
			this.add(f);
		}

		return f;
	}

	/**
	 * Closes frame on the desktop
	 * 
	 * @param frame
	 *            frame to close
	 */
	public void closeFrame(DataFrame frame) {
		if ((frame != null) && (frame.isClosable()))
			try {
				frame.setClosed(true);
			} catch (java.beans.PropertyVetoException ev) {
			}

		if (frame.isClosed()) {
			frame.dispose();
			frame = null;
		}
	}

	/**
	 * Closes the active (topmost) frame on the desktop
	 */
	public void closeTopFrame() {
		DataFrame active = getActiveFrame();

		closeFrame(active);
	}

	/**
	 * Closes all frames on the desktop
	 */
	public void closeAllFrames() {
		Vector frames = getDataFrames();
		for (int i = 0; i < frames.size(); i++) {
			closeFrame((DataFrame) frames.get(i));
		}
	}

	/**
	 * Cascades all frames
	 */
	public void cascade() {
		Vector frames = getDataFrames();

		int x = 0;
		int y = 0;
		int width = this.getWidth() / 2;
		int height = this.getHeight() / 2;

		for (int i = 0; i < frames.size(); i++) {
			if (!((DataFrame) frames.get(i)).isIcon()) {
				try {
					((DataFrame) frames.get(i)).setMaximum(false);
					((DataFrame) frames.get(i)).reshape(x, y, width, height);
				} catch (java.beans.PropertyVetoException ev) {
				}

				x += 20;
				y += 20;

				if (x + width > this.getWidth())
					x = 0;
				if (y + height > this.getHeight())
					y = 0;
			}
		}
	}

	/**
	 * Tiles all frames
	 */
	public void tile() {
		Vector frames = getDataFrames();

		int frameCount = 0;
		for (int i = 0; i < frames.size(); i++) {
			if (!((DataFrame) frames.get(i)).isIcon())
				frameCount++;
		}

		if (frameCount > 0) {
			int rows = (int) Math.sqrt(frameCount);
			int cols = frameCount / rows;
			int extra = frameCount % rows;

			int width = this.getWidth() / cols;
			int height = this.getHeight() / rows;
			int r = 0;
			int c = 0;
			for (int i = 0; i < frames.size(); i++) {
				if (!((DataFrame) frames.get(i)).isIcon()) {
					try {
						((DataFrame) frames.get(i)).setMaximum(false);
						((DataFrame) frames.get(i)).reshape(c * width, r
								* height, width, height);
					} catch (java.beans.PropertyVetoException ev) {
					}
					r++;
					if (r == rows) {
						r = 0;
						c++;
						if (c == cols - extra) {
							rows++;
							height = this.getHeight() / rows;
						}
					}
				}
			}
		}
	}

	/**
	 * Tiles all frames horizontaly
	 */
	public void tileHorizontaly() {
		Vector frames = getDataFrames();

		int frameCount = 0;
		for (int i = 0; i < frames.size(); i++) {
			if (!((DataFrame) frames.get(i)).isIcon())
				frameCount++;
		}

		if (frameCount > 0) {
			int x = 0;
			int y = 0;
			int width = this.getWidth() / frameCount;
			int height = this.getHeight();

			for (int i = 0; i < frames.size(); i++) {
				if (!((DataFrame) frames.get(i)).isIcon()) {
					try {
						((DataFrame) frames.get(i)).setMaximum(false);
						((DataFrame) frames.get(i))
								.reshape(x, y, width, height);
					} catch (java.beans.PropertyVetoException ev) {
					}
					x += width;

					if (x + width > this.getWidth())
						x = 0;
				}
			}
		}
	}

	/**
	 * Tiles all frames verticaly
	 */
	public void tileVerticaly() {
		Vector frames = getDataFrames();

		int frameCount = 0;
		for (int i = 0; i < frames.size(); i++) {
			if (!((DataFrame) frames.get(i)).isIcon())
				frameCount++;
		}

		if (frameCount > 0) {
			int x = 0;
			int y = 0;
			int width = this.getWidth();
			int height = this.getHeight() / frameCount;

			for (int i = 0; i < frames.size(); i++) {
				if (!((DataFrame) frames.get(i)).isIcon()) {
					try {
						((DataFrame) frames.get(i)).setMaximum(false);
						((DataFrame) frames.get(i))
								.reshape(x, y, width, height);
					} catch (java.beans.PropertyVetoException ev) {
					}

					y += height;

					if (y + height > this.getHeight())
						y = 0;
				}
			}
		}
	}

	/**
	 * Sets state for frame actions
	 * 
	 * @param state
	 *            state of the actions
	 */
	public void setWindowActions(boolean state) {
		windowMenu.setEnabled(state);

		this.firePropertyChange("frame_control", !state, state);
	}

	/**
	 * Getter for property windowMenu.
	 * 
	 * @return Value of property windowMenu.
	 */
	public WindowMenu getWindowMenu() {
		return windowMenu;
	}

	/**
	 * Getter for property preview.
	 * 
	 * @return Value of property preview.
	 */
	public PrintPreview getPreview() {
		return this.preview;
	}

	/**
	 * Getter for property tasks.
	 * 
	 * @return Value of property tasks.
	 */
	public TaskProgress getTasks() {
		return this.tasks;
	}

	/**
	 * Sets to true when a new frame has to be created at each record
	 * 
	 * @param create
	 *            true for new frame creation
	 */
	public void setCreateFrame(boolean create) {
		createFrame = create;
	}

	/**
	 * Returns true when a new frame has to be created at each record
	 * 
	 * @return true for new frame creation
	 */
	public boolean isCreateFrame() {
		return createFrame;
	}

	/**
	 * @param internalFrameEvent
	 */
	public void internalFrameActivated(
			javax.swing.event.InternalFrameEvent internalFrameEvent) {
		if (getDataFrames().size() > 0)
			setWindowActions(true);
	}

	/**
	 * @param internalFrameEvent
	 */
	public void internalFrameDeiconified(
			javax.swing.event.InternalFrameEvent internalFrameEvent) {
	}

	/**
	 * @param internalFrameEvent
	 */
	public void internalFrameDeactivated(
			javax.swing.event.InternalFrameEvent internalFrameEvent) {
	}

	/**
	 * @param internalFrameEvent
	 */
	public void internalFrameOpened(
			javax.swing.event.InternalFrameEvent internalFrameEvent) {
	}

	/**
	 * @param internalFrameEvent
	 */
	public void internalFrameIconified(
			javax.swing.event.InternalFrameEvent internalFrameEvent) {
	}

	/**
	 * @param internalFrameEvent
	 */
	public void internalFrameClosing(
			javax.swing.event.InternalFrameEvent internalFrameEvent) {
	}

	/**
	 * @param internalFrameEvent
	 */
	public void internalFrameClosed(
			javax.swing.event.InternalFrameEvent internalFrameEvent) {
		if (getDataFrames().size() < 1)
			setWindowActions(false);
	}

}