/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ChangeEvent;

import com.simulogic.dataview.actions.ViewAutoScaleAction;
import com.simulogic.dataview.actions.ViewFullScaleAction;
import com.simulogic.dataview.actions.ViewPropertiesAction;
import com.simulogic.dataview.actions.ViewZoomCursorsAction;
import com.simulogic.dataview.actions.ViewZoomInAction;
import com.simulogic.dataview.actions.ViewZoomOutAction;
import com.simulogic.dataview.cursors.CursorProxy;
import com.simulogic.dataview.cursors.CursorSlider;

/**
 * Class that visualizes the content of <code>DataStorage</code>
 * 
 * @see DataStorage
 */
public class DataView extends javax.swing.JPanel implements
		javax.swing.event.TreeModelListener, javax.swing.event.ChangeListener,
		java.awt.event.MouseListener, java.awt.event.MouseMotionListener,
		MouseWheelListener {

	/**
	 * Buffer for the plot image backstore
	 */
	private Image buffer;

	/**
	 * No clipping is used
	 */
	public static final int NO_CLIP = 0;

	/**
	 * Plotting is clipped to the view bounds
	 */
	public static final int VIEW_CLIP = 1;

	/**
	 * Plotting is clipped to getClipBounds()
	 */
	public static final int GRAPHICS_CLIP = 2;

	/**
	 * Plot clipping strategy
	 */
	private int clipping;

	private JScrollPane scroll;

	private DataStorage storage;

	private CursorProxy cursors;

	private JPopupMenu popup;

	private DataAxis xAxis;

	private DataAxis yAxis;

	private DataCorner xCorner;

	private DataCorner yCorner;

	private Rectangle2D.Double scale = new Rectangle2D.Double();

	private Rectangle2D.Double zoomBounds = new Rectangle2D.Double();

	private Point2D.Double zoomOrigin = new Point2D.Double();

	private Graphics2D zoomGraphics;

	private boolean zoomin = true;

	private Color scaleColor;

	private Color gridColor;

	private java.awt.geom.AffineTransform transform = new java.awt.geom.AffineTransform();

	private boolean locked;

	private boolean render;

	/**
	 * Default constructor
	 * 
	 * @param s
	 *            data storage
	 */
	public DataView(DataStorage s) {
		super();

		storage = s;

		setClipping(NO_CLIP);

		scroll = new JScrollPane(this,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);

		xAxis = new DataAxis(DataAxis.AXIS_HORIZONTAL, this);
		yAxis = new DataAxis(DataAxis.AXIS_VERTICAL, this);
		xCorner = new DataCorner(DataCorner.CORNER_HORIZONTAL, "", 2);
		yCorner = new DataCorner(DataCorner.CORNER_VERTICAL, "", 5);

		scroll.setCorner(ScrollPaneConstants.UPPER_RIGHT_CORNER, xCorner);
		scroll.setCorner(ScrollPaneConstants.UPPER_LEFT_CORNER, yCorner);
		scroll.setCorner(ScrollPaneConstants.LOWER_LEFT_CORNER, new DataCorner(
				DataCorner.CORNER_EMPTY, "", 0));

		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.addMouseWheelListener(this);

		((FlowLayout) this.getLayout()).setHgap(0);
		((FlowLayout) this.getLayout()).setVgap(0);

		setBackground(Color.black);
		setScaleColor(Color.white);
		setGridColor(new Color(72, 72, 72));

		scroll.getViewport().addChangeListener(this);

		storage.addTreeModelListener(this);

		cursors = new CursorProxy(this);
		this.add(cursors.getCursorSlider());

		this.setDoubleBuffered(false);
		cursors.getCursorSlider().setDoubleBuffered(false);
		scroll.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
	}

	/**
	 * Returns current DataStorage
	 * 
	 * @return DataStorage
	 */
	public DataStorage getStorage() {
		return storage;
	}

	/**
	 * Returns current scroll pane
	 * 
	 * @return scroll pane for the interior
	 */
	public JScrollPane getScrollPane() {
		return scroll;
	}

	/**
	 * Returns current X corner
	 * 
	 * @return X corner
	 */
	public DataCorner getXCorner() {
		return xCorner;
	}

	/**
	 * Returns current Y corner
	 * 
	 * @return Y corner
	 */
	public DataCorner getYCorner() {
		return yCorner;
	}

	/**
	 * Returns current X axis
	 * 
	 * @return X axis
	 */
	public DataAxis getXAxis() {
		return xAxis;
	}

	/**
	 * Returns current Y axis
	 * 
	 * @return Y axis
	 */
	public DataAxis getYAxis() {
		return yAxis;
	}

	/**
	 * Returns cursors
	 * 
	 * @return cursors
	 */
	public CursorProxy getCursors() {
		return cursors;
	}

	/**
	 * Returns current transform
	 * 
	 * @return transform
	 */
	public AffineTransform getTransform() {
		return transform;
	}

	/**
	 * @param clipping
	 *            The clipping to set.
	 */
	protected void setClipping(int clipping) {
		this.clipping = clipping;
	}

	/**
	 * @return Returns the clipping.
	 */
	protected int getClipping() {
		return clipping;
	}

	/**
	 * Returns the plotting clip based on the current clipping strategy
	 * 
	 * @return current plot clip
	 * @param g2
	 *            graphics to find the clip for
	 */
	protected java.awt.geom.Rectangle2D.Double getClip(Graphics2D g2) {
		java.awt.geom.Rectangle2D.Double clip = new java.awt.geom.Rectangle2D.Double();

		switch (getClipping()) {
		case 0:
			clip.setRect(storage.getLimitRectangle());
			break;
		case 1:
			clip.setRect(getScale());
			break;
		case 2:
			Area area = new Area(g2.getClipBounds());
			try {
				area.transform(getTransform().createInverse());
			} catch (Exception e) {
			}
			clip.setRect(area.getBounds());
			break;
		}

		return clip;
	}

	/**
	 * Plots grid lines based on tick data from axis
	 * 
	 * @param g2
	 *            graphics
	 */
	protected final void plotGrid(Graphics2D g2) {
		java.awt.geom.Rectangle2D.Double clip = new java.awt.geom.Rectangle2D.Double();
		clip.setFrame(getScale());

		GeneralPath gp = new GeneralPath();
		float[] dashPattern = { 2, 2, 2, 2 };

		double val = 0.0;

		if (xAxis.isShowGrid()) {
			g2.setColor(gridColor);
			g2.setStroke(new java.awt.BasicStroke(1.0f, BasicStroke.CAP_BUTT,
					BasicStroke.JOIN_MITER, 1.0f, dashPattern, 0));
			for (int i = 0; i < xAxis.getTicks().size(); i++) {
				val = (((Double) xAxis.getTicks().get(i)).doubleValue());
				gp.moveTo((float) val, (float) clip.y);
				gp.lineTo((float) val, (float) (clip.y - clip.height));
			}
			g2.draw(gp.createTransformedShape(transform));
		}

		gp.reset();

		if (xAxis.isZeroCross()) {
			g2.setColor(xAxis.getBaseColor());
			g2.setStroke(new java.awt.BasicStroke(1.0f));
			gp.moveTo((float) 0.0, (float) clip.y);
			gp.lineTo((float) 0.0, (float) (clip.y - clip.height));
			g2.draw(gp.createTransformedShape(transform));
		}

		gp.reset();

		if (yAxis.isShowGrid()) {
			g2.setColor(gridColor);
			g2.setStroke(new java.awt.BasicStroke(1.0f, BasicStroke.CAP_BUTT,
					BasicStroke.JOIN_MITER, 1.0f, dashPattern, 0));
			for (int j = 0; j < yAxis.getTicks().size(); j++) {
				val = (((Double) yAxis.getTicks().get(j)).doubleValue());
				gp.moveTo((float) clip.x, (float) val);
				gp.lineTo((float) (clip.x + clip.width), (float) val);
			}
			g2.draw(gp.createTransformedShape(transform));
		}

		gp.reset();

		if (yAxis.isZeroCross()) {
			g2.setColor(yAxis.getBaseColor());
			g2.setStroke(new java.awt.BasicStroke(1.0f));
			gp.moveTo((float) clip.x, (float) 0.0);
			gp.lineTo((float) (clip.x + clip.width), (float) 0.0);
			g2.draw(gp.createTransformedShape(transform));
		}
	}

	/**
	 * Plots data (as a set of channels) using <code>plotChannel</code>
	 * 
	 * @param g2
	 *            graphics
	 */
	protected final void plotData(Graphics2D g2) {
		com.simulogic.dataview.DataGroup group = null;
		com.simulogic.dataview.DataChannel xchannel = null;
		com.simulogic.dataview.DataChannel ychannel = null;

		java.awt.geom.Rectangle2D.Double clip = null;

		try {
			clip = getClip(g2);
		} catch (Exception e) {
			return;
		}

		double xmin = clip.getX();
		double xmax = clip.getX() + clip.getWidth();

		try {
			for (int i = 0; i < storage.getGroupsSize(); i++) {
				group = storage.getGroup(i);
				xchannel = storage.getChannel(i, group.getXChannel());

				for (int j = 0; j < storage.getChannelsSize(i); j++) {
					if (j != group.getXChannel()) {
						ychannel = storage.getChannel(i, j);
						if (ychannel != storage.getSelectedChannel())
							ychannel.getPlot().plotChannel(g2, xchannel,
									ychannel, xmin, xmax, transform);
					}
				}
			}

			ychannel = storage.getSelectedChannel();
			if (ychannel != null) {
				xchannel = storage.getXChannel(ychannel);
				if ((xchannel != null) && (xchannel != ychannel))
					ychannel.getPlot().plotChannel(g2, xchannel, ychannel,
							xmin, xmax, transform);
			}
		} catch (Exception e) {
		}
	}

	/**
	 * Renders plot data
	 */
	public final void renderPlot() {
		Point org = getScrollPane().getViewport().getViewPosition();
		Dimension size = getScrollPane().getViewport().getExtentSize();

		if ((size.width <= 0) || (size.height <= 0))
			return;

		GraphicsConfiguration gc = getGraphicsConfiguration();

		if ((gc != null) && (this.isShowing()) && (!locked)) {
			if ((buffer == null) || (buffer.getWidth(null) != size.width)
					|| (buffer.getHeight(null) != size.height))
				buffer = gc.createCompatibleImage(size.width, size.height);

			Graphics2D g2 = (Graphics2D) buffer.getGraphics();
			g2.setBackground(this.getBackground());
			g2.clearRect(0, 0, size.width, size.height);
			g2.translate(-org.getX(), -org.getY());

			plotGrid(g2);
			plotData(g2);
			g2.dispose();
			render = false;
		}

		if (buffer != null) {
			repaint();
			buffer.flush();
		}
	}

	/**
	 * Selects a channel according to given coordinates
	 * 
	 * @param x
	 *            x coordinate
	 * @param y
	 *            y coordinate
	 */
	protected void selectChannel(double x, double y) {
		for (int i = 0; i < storage.getGroupsSize(); i++) {
			com.simulogic.dataview.DataGroup group = storage.getGroup(i);
			com.simulogic.dataview.DataChannel xchannel = storage.getChannel(i,
					group.getXChannel());

			for (int j = 0; j < storage.getChannelsSize(i); j++) {
				DataChannel ychannel = storage.getChannel(i, j);
				if (!storage.isXChannel(ychannel)) {
					if (ychannel.getPlot().isChannelPoint(x, y, xchannel,
							ychannel, transform)) {
						fireSelectChannel(storage.getChannelNode(i, j));
						return;
					}
				}
			}
		}
	}

	/**
	 * Overrides correspondent JComponent method
	 * 
	 * @param g
	 *            view graphics
	 */
	public void paintComponent(java.awt.Graphics g) {
		super.paintComponent(g);

		if (render)
			renderPlot();

		Graphics2D g2 = (Graphics2D) g;

		if (!g2.getClipBounds().isEmpty()) {
			if (buffer != null) {
				g2.drawImage(buffer, scroll.getViewport().getViewPosition().x,
						scroll.getViewport().getViewPosition().y, this);
			}
		}
	}

	/**
	 * Overrides correspondent JComponent method
	 * 
	 * @param g
	 *            view graphics
	 */
	public void printComponent(java.awt.Graphics g) {
		super.paintComponent(g);

		Graphics2D g2 = (Graphics2D) g;

		plotGrid(g2);
		plotData(g2);

		g2.dispose();
	}

	/**
	 * Returns current bounds of this view in user space
	 * 
	 * @return view scale
	 */
	public Rectangle2D.Double getScale() {
		return scale;
	}

	/**
	 * Changes the scale for the data
	 * 
	 * @param xorig
	 *            new x axis start value
	 * @param yorig
	 *            new y axis start value
	 * @param xsize
	 *            new x axis interval
	 * @param ysize
	 *            new y axis interval
	 */
	public void setScale(double xorig, double yorig, double xsize, double ysize) {
		double threshold = 0.005;

		xsize = Math.abs(xsize);
		ysize = Math.abs(ysize);

		if ((Double.isNaN(xsize)) || (Double.isInfinite(xsize))
				|| (xsize < threshold))
			xsize = threshold;
		if ((Double.isNaN(ysize)) || (Double.isInfinite(ysize))
				|| (ysize < threshold))
			ysize = threshold;

		scale.setRect(xorig, yorig, xsize, ysize);

		xAxis.stateChanged(new ChangeEvent(scroll.getColumnHeader()));
		yAxis.stateChanged(new ChangeEvent(scroll.getRowHeader()));

		changeLimits();
		cursors.stateChanged(new ChangeEvent(scroll.getViewport()));
	}

	/**
	 * Changes the scale for the data
	 * 
	 * @param rsc
	 *            area in the <code>DataView</code>
	 */
	public void setScale(Rectangle2D.Double rsc) {
		setScale(rsc.x, rsc.y, rsc.width, rsc.height);
	}

	/**
	 * Zooms in to a range between cursors 1..2
	 */
	public void zoomToCursors() {
		if ((cursors != null) && cursors.getCursorSlider().isVisible()) {
			double origin = cursors.getCursorSlider().getValueAt(0);
			if (cursors.getCursorSlider().getValueAt(1) < cursors
					.getCursorSlider().getValueAt(0))
				origin = cursors.getCursorSlider().getValueAt(1);

			setScale(origin, getScale().getY(), Math.abs(cursors
					.getCursorSlider().getValueAt(1)
					- cursors.getCursorSlider().getValueAt(0)), getScale()
					.getHeight());
		}
	}

	/**
	 * Zooms around center by a certain factor
	 * 
	 * @param factor
	 *            zoom factor. If positive, zooms in; if negative, zooms out
	 */
	public void zoomCentered(double factor) {
		double mult_f = factor;

		if (factor >= 0) {
			mult_f = 1.0 / factor;
		}

		setScale((getScale().getX() + (getScale().getWidth() / 2.0))
				- Math.abs((getScale().getWidth() * mult_f) / 2.0), (getScale()
				.getY() - (getScale().getHeight() / 2.0))
				+ Math.abs((getScale().getHeight() * mult_f) / 2.0), getScale()
				.getWidth()
				* mult_f, getScale().getHeight() * mult_f);
	}

	/**
	 * Rescales the data to full scale
	 */
	public void fullScale() {
		setScale(storage.getLimitRectangle());
	}

	/**
	 * Changes the scrolling position and scale of the given interior to scale
	 * and position of this interior
	 * 
	 * @param intr
	 *            <code>DataView</code> to be synced with this interior
	 */
	public void syncScale(DataView intr) {
		intr.setScale(new Rectangle2D.Double(getScale().getX(), getScale()
				.getY(), getScale().getWidth(), getScale().getHeight()));
	}

	/**
	 * Getter for property scaleColor.
	 * 
	 * @return Value of property scaleColor.
	 */
	public java.awt.Color getScaleColor() {
		return scaleColor;
	}

	/**
	 * Setter for property scaleColor.
	 * 
	 * @param scaleColor
	 *            New value of property scaleColor.
	 */
	public void setScaleColor(java.awt.Color scaleColor) {
		this.scaleColor = scaleColor;
	}

	/**
	 * Getter for property gridColor.
	 * 
	 * @return Value of property gridColor.
	 */
	public Color getGridColor() {
		return gridColor;
	}

	/**
	 * Setter for property gridColor.
	 * 
	 * @param gridColor
	 *            New value of property gridColor.
	 */
	public void setGridColor(Color gridColor) {
		this.gridColor = gridColor;
	}

	/**
	 * Creates and displays dialog for changing view propertis
	 */
	public void showPropertiesDialog() {
		new com.simulogic.dataview.ViewProperties(null, this).setVisible(true);
	}

	/**
	 * Fires PropertyChange event to inform listeners about channel selection
	 * 
	 * @param channel
	 *            new selected channel
	 */
	public void fireSelectChannel(Object channel) {
		firePropertyChange("select_channel", null, channel);
	}

	private boolean changeTransform() {
		double ddx = scroll.getViewport().getExtentSize().getWidth()
				/ getScale().getWidth();
		double ddy = scroll.getViewport().getExtentSize().getHeight()
				/ getScale().getHeight();

		if (!Double.isInfinite(ddx) && !Double.isNaN(ddx)
				&& !Double.isInfinite(ddy) && !Double.isNaN(ddy)) {
			if ((transform.getScaleX() != ddx)
					|| (transform.getScaleY() != -ddy)) {
				transform.setToScale(ddx, -ddy);
				transform.translate(-Math.min(storage.getLimitRectangle()
						.getX(), getScale().getX()), -Math.max(storage
						.getLimitRectangle().getY(), getScale().getY()));

				return true;
			}
		}
		return false;
	}

	private void changeLimits() {
		try {
			Point2D.Double position = new Point2D.Double();

			if ((storage == null) || (storage.getGroupsSize() == 0))
				return;

			boolean update = changeTransform();
			double xs = Math.max(storage.getLimitRectangle().getWidth(),
					getScale().getWidth());
			double ys = Math.max(storage.getLimitRectangle().getHeight(),
					getScale().getHeight());

			int hl = (int) Math.ceil(Math.abs(xs * transform.getScaleX()));
			int vl = (int) Math.ceil(Math.abs(ys * transform.getScaleY()));

			if ((hl == 0) || (hl == Integer.MAX_VALUE)
					|| (hl == Integer.MIN_VALUE))
				hl = scroll.getViewport().getWidth();

			if ((vl == 0) || (vl == Integer.MAX_VALUE)
					|| (vl == Integer.MIN_VALUE))
				vl = scroll.getViewport().getHeight();

			if ((hl != getPreferredSize().width)
					|| (vl != getPreferredSize().height) || update) {
				boolean l = locked;
				locked = true;

				setPreferredSize(new java.awt.Dimension(hl, vl));

				this.revalidate();

				transform.transform(new Point2D.Double(scale.getX(), scale
						.getY()), position);

				scroll.getViewport().setViewPosition(
						new java.awt.Point((int) position.x, (int) position.y));

				scrollRectToVisible(new java.awt.Rectangle((int) position.x,
						(int) position.y, hl, vl));

				locked = l;
				render = true;
				repaint();
			} else {
				transform.inverseTransform(new Point2D.Double(scroll
						.getViewport().getViewPosition().getX(), scroll
						.getViewport().getViewPosition().getY()), position);

				if ((scale.x != position.getX())
						|| (scale.y != position.getY())) {
					scale.x = position.getX();
					scale.y = position.getY();

					xAxis
							.stateChanged(new ChangeEvent(scroll
									.getColumnHeader()));
					yAxis.stateChanged(new ChangeEvent(scroll.getRowHeader()));

					render = true;
					repaint();
				}
			}

		} catch (Exception e) {
		}
	}

	private void flipFrame(Rectangle2D.Double b) {
		drawFrame(zoomBounds);
		zoomBounds.setRect(b);
		drawFrame(b);
	}

	private void drawFrame(Rectangle2D.Double b) {
		if (!zoomin) {
			Rectangle2D.Double outBounds = new Rectangle2D.Double(zoomOrigin.x
					- scroll.getViewport().getViewRect().width / 20.0,
					zoomOrigin.y - scroll.getViewport().getViewRect().height
							/ 20.0,
					scroll.getViewport().getViewRect().width / 10.0, scroll
							.getViewport().getViewRect().height / 10.0);
			b.setRect(outBounds.x - b.width, outBounds.y - b.height,
					outBounds.width + b.width * 2.0, outBounds.height
							+ b.height * 2.0);
		}
		try {
			zoomGraphics.draw(b);
		} catch (Exception e) {
		}
	}

	private void zoom(int x, int y) {
		if ((zoomOrigin.y < y) && (zoomOrigin.x < x)) {
			if (!zoomin) {
				drawFrame(zoomBounds);
				zoomBounds.setRect(zoomOrigin.x, zoomOrigin.y, -1, -1);
				drawFrame(zoomBounds);
				zoomBounds.setRect(zoomOrigin.x, zoomOrigin.y, -1, -1);
			}

			zoomin = true;
			flipFrame(new Rectangle2D.Double(zoomOrigin.x, zoomOrigin.y, x
					- zoomOrigin.x, y - zoomOrigin.y));
		}

		if ((zoomOrigin.y > y) && (zoomOrigin.x > x)) {
			if (zoomin) {
				drawFrame(zoomBounds);
				zoomBounds.setRect(zoomOrigin.x, zoomOrigin.y, -1, -1);
				drawFrame(zoomBounds);
				zoomBounds.setRect(zoomOrigin.x, zoomOrigin.y, -1, -1);
			}

			zoomin = false;
			flipFrame(new Rectangle2D.Double(x, y, zoomOrigin.x - x,
					zoomOrigin.y - y));
		}
	}

	private JPopupMenu createPopup() {
		JMenuItem item;
		if (popup == null) {
			try {
				popup = new JPopupMenu();
				item = popup.add(new ViewFullScaleAction(this));
				item.setAccelerator(KeyStroke.getKeyStroke('1', 2));
				popup.add(new ViewAutoScaleAction(this));
				popup.add(new ViewZoomInAction(this));
				popup.add(new ViewZoomOutAction(this));
				popup.add(new ViewZoomCursorsAction(this));
				popup.addSeparator();
				popup.add(cursors.createPopup());
				item = popup.add(new ViewPropertiesAction(this));
				item.setAccelerator(KeyStroke.getKeyStroke(
						java.awt.event.KeyEvent.VK_ENTER, 2));
			} catch (Exception e) {
			}
		}
		cursors.createPopup();
		return popup;
	}

	// / --- interfaces

	/**
	 * @param evt
	 */
	public void mouseClicked(MouseEvent evt) {
	}

	/**
	 * @param evt
	 */
	public void mousePressed(MouseEvent evt) {
		if (evt.getSource() == cursors.getCursorSlider())
			evt = javax.swing.SwingUtilities.convertMouseEvent(cursors
					.getCursorSlider(), evt, this);

		if (evt.isPopupTrigger()) {
			createPopup().show(evt.getComponent(), evt.getX(), evt.getY());
			return;
		}

		zoomGraphics = (Graphics2D) getGraphics();

		if (evt.isControlDown()) {
			zoomOrigin.setLocation(evt.getX(), evt.getY());
			zoomBounds.setRect(zoomOrigin.x, zoomOrigin.y, -1, -1);

			zoomGraphics.setXORMode(getBackground());
			zoomGraphics.setColor(scaleColor);

			xAxis.mousePressed(evt);
			yAxis.mousePressed(evt);
		}
	}

	/**
	 * @param evt
	 */
	public void mouseReleased(MouseEvent evt) {
		if (evt.getSource() == cursors.getCursorSlider())
			evt = javax.swing.SwingUtilities.convertMouseEvent(cursors
					.getCursorSlider(), evt, this);

		if (evt.isPopupTrigger()) {
			createPopup().show(evt.getComponent(), evt.getX(), evt.getY());
			return;
		}

		Rectangle2D.Double r = new Rectangle2D.Double(zoomBounds.x,
				zoomBounds.y, zoomBounds.width, zoomBounds.height);
		drawFrame(zoomBounds);
		zoomBounds.setRect(zoomOrigin.x, zoomOrigin.y, -1, -1);
		drawFrame(zoomBounds);
		zoomBounds.setRect(zoomOrigin.x, zoomOrigin.y, -1, -1);

		xAxis.clearFrame();
		yAxis.clearFrame();

		try {
			zoomGraphics.setPaintMode();
			zoomGraphics.dispose();
		} catch (Exception e) {
		}

		if (evt.isShiftDown())
			cursors.bringCursorsHere();

		if (evt.isControlDown()) {
			if (evt.getClickCount() == 2) {
				fullScale();
			} else {
				if ((r.width > 1) || (r.height > 1)) {
					try {
						if (zoomin) {
							/*
							 * Zoom in strategy: Take coordinates from the
							 * zoomin box (they are in device space) and convert
							 * them to the user space (using inverseTransform).
							 * Then set scale using the converted rectangle.
							 */
							Point2D.Double a = new Point2D.Double();
							Point2D.Double b = new Point2D.Double();

							transform.inverseTransform(new Point2D.Double(r.x,
									r.y), a);
							b.x = r.width / transform.getScaleX();
							b.y = r.height / transform.getScaleY();

							setScale(a.x, a.y, b.x, b.y);
						} else {
							/*
							 * Zoom out strategy: First, we need to determine
							 * the new view width and height. This is done by
							 * estimating the ratio between the inner zoomout
							 * box (it represents the current view size) scaled
							 * down by the factor of 10. This ratio is then
							 * being multiplied by the current view width and
							 * height. After this we can set the view new center
							 * point, which should be equal to the center point
							 * of the zoomout box (converted to the user space).
							 */
							Point2D.Double a = new Point2D.Double();
							Point2D.Double b = new Point2D.Double();

							// ratios
							double rx = ((r.width * 2) / (scroll.getViewport()
									.getViewRect().width / 10.0)) + 1;
							double ry = ((r.height * 2) / (scroll.getViewport()
									.getViewRect().height / 10.0)) + 1;

							// new width and height
							b.x = scale.width * rx;
							b.y = scale.height * ry;

							// new zoom origin based on zoomout box center
							// point
							transform.inverseTransform(new Point2D.Double(r.x
									+ r.width / 2, r.y - r.height / 2), a);

							a.x = a.x - b.x / 2;
							a.y = a.y + b.y / 2;

							setScale(a.x, a.y, b.x, b.y);
						}
					} catch (Exception e) {
					}
				}
			}
		} else {
			selectChannel(evt.getX(), evt.getY());
		}
	}

	/**
	 * @param evt
	 */
	public void mouseDragged(MouseEvent evt) {
		if ((evt.getSource() == cursors.getCursorSlider())
				&& (!evt.isControlDown())) {
			Rectangle r = new Rectangle(evt.getX(), evt.getY(), 1, 1);
			((CursorSlider) evt.getSource()).scrollRectToVisible(r);

			evt = javax.swing.SwingUtilities.convertMouseEvent(cursors
					.getCursorSlider(), evt, this);
		}

		if (evt.isControlDown()) {
			xAxis.mouseDragged(evt);
			yAxis.mouseDragged(evt);

			zoom(evt.getX(), evt.getY());
		}
	}

	/**
	 * @param evt
	 */
	public void mouseMoved(MouseEvent evt) {
	}

	/**
	 * @param evt
	 */
	public void mouseEntered(MouseEvent evt) {
	}

	/**
	 * @param evt
	 */
	public void mouseExited(MouseEvent evt) {
	}

	/**
	 * @param evt
	 */
	public void mouseWheelMoved(MouseWheelEvent evt) {
		double zoomFactor = 1.02;

		int notches = evt.getWheelRotation();

		if (evt.isControlDown()) {
			if (notches < 0) {
				yAxis.zoomCentered(-zoomFactor);
			} else {
				yAxis.zoomCentered(zoomFactor);
			}
		}

		if (evt.isAltDown()) {
			if (notches < 0) {
				xAxis.zoomCentered(-zoomFactor);
			} else {
				xAxis.zoomCentered(zoomFactor);
			}
		}
	}

	/**
	 * @param evt
	 */
	public void stateChanged(javax.swing.event.ChangeEvent evt) {
		if (evt.getSource() == scroll.getViewport())
			changeLimits();
	}

	/**
	 * @param evt
	 */
	public void treeStructureChanged(javax.swing.event.TreeModelEvent evt) {
	}

	/**
	 * @param evt
	 */
	public void treeNodesInserted(javax.swing.event.TreeModelEvent evt) {
		if ((evt.getSource() == storage)
				&& (evt.getTreePath().getPathCount() > 1)
				&& (storage.getGroupsSize() > 0)) {
			xCorner.setCorner(storage, evt);
			yCorner.setCorner(storage, evt);

			locked = true;
			fullScale();
			locked = false;

			render = true;
			repaint();
		}
	}

	/**
	 * @param evt
	 */
	public void treeNodesRemoved(javax.swing.event.TreeModelEvent evt) {
		if ((evt.getSource() == storage)
				&& (evt.getTreePath().getPathCount() > 1)
				&& (storage.getGroupsSize() > 0)) {
			xCorner.setCorner(storage, evt);
			yCorner.setCorner(storage, evt);

			locked = true;
			fullScale();
			locked = false;

			render = true;
			repaint();
		}
	}

	/**
	 * @param evt
	 */
	public void treeNodesChanged(javax.swing.event.TreeModelEvent evt) {
		renderPlot();
	}

}