/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.awt.print.Book;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

import javax.help.CSH;
import javax.print.PrintService;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.ScrollPaneConstants;
import javax.swing.UIManager;

import com.simulogic.dataview.legend.LegendTreeRenderer;

/**
 * Class that displays the print preview frame
 */
public class PrintPreview extends javax.swing.JDialog implements
		java.awt.print.Printable {

	private DataDesktop desktop;

	private JScrollPane printScroll;

	private PrintRequestAttributeSet ps;

	private PrinterJob pj;

	private PageFormat pf;

	private JComboBox zoom;

	private JButton closeButton;

	private PagePreview pagePreview;

	private DataFrame workFrame;

	private PrintOptions options;

	private String[] scales = { "25%", "33%", "50%", "100%", "150%", "200%" };

	/**
	 * Creates new PrintPreview
	 * 
	 * @param parent
	 *            parent frame
	 * @param desktop
	 *            data desktop
	 */
	public PrintPreview(java.awt.Frame parent, DataDesktop desktop) {
		super();
		this.desktop = desktop;
		this.getContentPane().setLayout(new java.awt.BorderLayout());
		this.setTitle(ResourceManager.getResource("Print_preview"));

		ps = new HashPrintRequestAttributeSet();
		pj = PrinterJob.getPrinterJob();
		pj.setPrintable(this);
		pf = pj.defaultPage();

		PrintService[] services = PrinterJob.lookupPrintServices();

		if (services.length > 0) {
			try {
				pj.setPrintService(services[0]);
				pf = pj.defaultPage();
			} catch (PrinterException pe) {
			}
		}

		pf.setOrientation(PageFormat.LANDSCAPE);

		this.getContentPane().add(buildPrintToolBar(),
				java.awt.BorderLayout.NORTH);
		printScroll = new JScrollPane();
		this.getContentPane().add(printScroll, java.awt.BorderLayout.CENTER);

		this.options = new PrintOptions(this, this);
	}

	private void closeButtonActionPerformed() {
		this.setVisible(false);
	}

	/**
	 * Displays PageFormat dialog
	 */
	public void setPageFormat() {
		PageFormat p = pj.pageDialog(ps);
		if (p != pf) {
			pf = p;
			updatePreview();
		}
	}

	/**
	 * Returns zoom ComboBox
	 * 
	 * @return zoom box
	 */
	public JComboBox getZoomBox() {
		return zoom;
	}

	/**
	 * Prints the content of the frame
	 * 
	 * @param g
	 *            current graphics content
	 * @param pf
	 *            current page format
	 * @param pi
	 *            current page index
	 * @return number of page to print
	 */
	public synchronized int print(java.awt.Graphics g,
			java.awt.print.PageFormat pf, int pi) {
		if (pi != 0)
			return java.awt.print.Printable.NO_SUCH_PAGE;
		try {
			java.awt.Graphics2D g2 = (java.awt.Graphics2D) g;
			g2.translate(pf.getImageableX(), pf.getImageableY());

			pagePreview.getPrintFrame().getContentPane().printAll(g2);

			g2.dispose();
		} catch (Exception e) {
		}
		return java.awt.print.Printable.PAGE_EXISTS;
	}

	/**
	 * Starts the printing
	 * 
	 * @throws PrinterException
	 *             printing problems
	 */
	public void doPrint() throws PrinterException {
		pf = pj.validatePage(pf);
		setBook();
		pj.print(ps);
	}

	/**
	 * Shows print dialog
	 * 
	 * @return true if OK is pressed
	 */
	public boolean showPrintDialog() {
		return pj.printDialog(ps);
	}

	/**
	 * Displays a preview frame
	 */
	public void showPreview() {
		workFrame = desktop.getActiveFrame();

		options.setView(this);

		updatePreview();
	}

	/**
	 * Updates a preview frame
	 */
	public void updatePreview() {
		setPreview(pf);
	}

	public PrintOptions getOptions() {
		return options;
	}

	/**
	 * Returns frame that is being used for generating print preview
	 * 
	 * @return work frame
	 */
	public DataFrame getWorkFrame() {
		return workFrame;
	}

	private void setBook() {
		updatePreview();

		Book book = new Book();
		book.append(this, pf);
		pj.setPageable(book);
	}

	private JToolBar buildPrintToolBar() {
		JToolBar printToolBar = new javax.swing.JToolBar();
		printToolBar.setLayout(new javax.swing.BoxLayout(printToolBar, 0));

		closeButton = new JButton(ResourceManager.getResource("Close"));
		closeButton.setMnemonic('c');
		closeButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(java.awt.event.ActionEvent evt) {
				closeButtonActionPerformed();
			}
		});

		zoom = new JComboBox(scales);
		ActionListener zoomAction = new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				String str = zoom.getSelectedItem().toString();
				str = str.trim();
				if (str.endsWith("%"))
					str = str.substring(0, str.length() - 1);
				str = str.trim();
				double scale = 0.0;
				try {
					scale = Double.parseDouble(str);
				} catch (NumberFormatException ex) {
					return;
				}

				pagePreview.setZoom(scale / 100.0);

				PreviewContainer printContainer = new PreviewContainer();
				printContainer.add(pagePreview.getPreviewPanel());
				printScroll.setViewportView(printContainer);
			}
		};

		javax.swing.AbstractButton b = printToolBar
				.add(new com.simulogic.dataview.actions.PageFormatAction(desktop));
		CSH.setHelpIDString(b, "Commands.Page_Setup");
		b= printToolBar.add(new com.simulogic.dataview.actions.PrintOptAction(
				desktop));
		CSH.setHelpIDString(b, "Commands.Print_Options");
		printToolBar.addSeparator();
		b = printToolBar.add(new com.simulogic.dataview.actions.PrintResAction(
				desktop));
		CSH.setHelpIDString(b, "Commands.Print_Results");
		printToolBar.addSeparator();
		printToolBar.add(new com.simulogic.dataview.actions.PrevZoomOutAction(
				desktop));
		printToolBar.add(zoom);
		printToolBar.add(new com.simulogic.dataview.actions.PrevZoomInAction(
				desktop));
		printToolBar.add(Box.createHorizontalGlue());
		printToolBar.add(closeButton);

		zoom.addActionListener(zoomAction);
		zoom.setMaximumSize(zoom.getPreferredSize());
		zoom.setEditable(true);

		return printToolBar;
	}

	private void setPreview(PageFormat p) {
		try {
			pagePreview = new PagePreview(p);
			PreviewContainer printContainer = new PreviewContainer();
			printContainer.add(pagePreview.getPreviewPanel());
			printScroll.setViewportView(printContainer);
			zoom.setSelectedItem("100%");
		} catch (Exception e) {
		}
		this.pack();
	}

	class PreviewContainer extends JPanel {

		/**
		 * Horizontal gap
		 */
		protected int H_GAP = 15;

		/**
		 * Vertical gap
		 */
		protected int V_GAP = 15;

		PreviewContainer() {
			this.setLayout(new FlowLayout(FlowLayout.CENTER, H_GAP, V_GAP));
		}
	}

	class PagePreview extends JPanel implements java.awt.event.MouseListener {

		private PageFormat format;

		private JFrame printFrame;

		private DataView dataView, printView;

		private JLabel previewLabel;

		private double zoom = 1.0;

		/**
		 * Page preview
		 * 
		 * @param p
		 *            page format
		 */
		public PagePreview(PageFormat p) {
			format = p;
			setBackground(Color.white);
			setBorder(new javax.swing.border.MatteBorder(1, 1, 4, 4,
					Color.black));
			this.setLayout(new java.awt.GridBagLayout());

			dataView = workFrame.getSelectedContent().getView();

			String xcrnr = "";
			String ycrnr = "";

			try {
				xcrnr = dataView.getXCorner().getText();
				ycrnr = dataView.getYCorner().getText();
			} catch (Exception e) {
			}

			printView = new DataView(new DataStorage());

			if (dataView.isDisplayable()) {
				printView = new DataView(dataView.getStorage());
			}

			printView.getCursors().getCursorSlider().setVisible(false);
			dataView.syncScale(printView);

			printView.getXCorner().setText(xcrnr);
			printView.getYCorner().setText(ycrnr);
			printView.getXCorner().setBackground(Color.lightGray);
			printView.getXCorner().setForeground(Color.black);
			printView.getYCorner().setBackground(Color.lightGray);
			printView.getYCorner().setForeground(Color.black);

			printView.getXAxis().setBackground(Color.lightGray);
			printView.getXAxis().setForeground(Color.black);
			printView.getXAxis().setBaseColor(Color.black);
			printView.getXAxis().setTicksColor(Color.black);
			printView.getXAxis().setFont(dataView.getXAxis().getFont());
			printView.getXAxis()
					.setZeroCross(dataView.getXAxis().isZeroCross());

			printView.getYAxis().setBackground(Color.lightGray);
			printView.getYAxis().setForeground(Color.black);
			printView.getYAxis().setBaseColor(Color.black);
			printView.getYAxis().setTicksColor(Color.black);
			printView.getYAxis().setFont(dataView.getYAxis().getFont());
			printView.getYAxis()
					.setZeroCross(dataView.getYAxis().isZeroCross());

			printView.setBackground(Color.white);
			printView.setGridColor(Color.lightGray);
			printView.getScrollPane().setHorizontalScrollBarPolicy(
					ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
			printView.getScrollPane().setVerticalScrollBarPolicy(
					ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

			printFrame = new JFrame();
			printFrame.addNotify();

			JPanel printPanel = new JPanel(new BorderLayout());
			printPanel.setBackground(Color.white);

			JPanel labelPanel = new JPanel(new BorderLayout());
			labelPanel.setBackground(Color.white);

			if (options.isPrintHeader()) {
				javax.swing.JLabel titleLabel = new javax.swing.JLabel(options
						.getHeader(), javax.swing.SwingConstants.LEFT);
				titleLabel.setForeground(Color.black);
				labelPanel.add(titleLabel, java.awt.BorderLayout.WEST);
			}

			if (options.isPrintDate()) {
				javax.swing.JLabel dateLabel = new javax.swing.JLabel(options
						.getDate(), javax.swing.SwingConstants.RIGHT);
				dateLabel.setForeground(Color.black);
				labelPanel.add(dateLabel, java.awt.BorderLayout.EAST);
			}

			printPanel.add(labelPanel, java.awt.BorderLayout.NORTH);

			if (options.isPrintComment()) {
				javax.swing.JLabel commentLabel = new javax.swing.JLabel(
						options.getComment(), javax.swing.SwingConstants.CENTER);
				commentLabel.setForeground(Color.black);
				printPanel.add(commentLabel, java.awt.BorderLayout.SOUTH);
			}

			printPanel.add(printView.getScrollPane(),
					java.awt.BorderLayout.CENTER);

			if (options.isPrintLegend()) {
				printPanel.add(getLegendPanel(dataView),
						java.awt.BorderLayout.EAST);
			}

			printPanel.setPreferredSize(new java.awt.Dimension((int) Math
					.abs(format.getImageableWidth()), (int) Math.abs(format
					.getImageableHeight())));

			java.awt.GridBagConstraints gridBagConstraints = new java.awt.GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;

			previewLabel = new JLabel();
			previewLabel.setPreferredSize(printFrame.getContentPane()
					.getPreferredSize());

			this.add(previewLabel, gridBagConstraints);

			printFrame.getContentPane().add(printPanel);
			printFrame.pack();

			this.addMouseListener(this);
		}

		private Box getLegendPanel(DataView v) {
			javax.swing.JTree tree = new javax.swing.JTree(v.getStorage());

			Box legendPanel = Box.createVerticalBox();

			if (options.isPrintLegend()) {
				legendPanel.add(Box.createVerticalStrut(10));

				JLabel l = new JLabel(options.getLegend(),
						javax.swing.SwingConstants.CENTER);
				l.setAlignmentX(0.5f);
				l.setAlignmentY(0.5f);
				legendPanel.add(l);
				legendPanel.add(Box.createVerticalStrut(10));
			}

			for (int i = 0; i < v.getStorage().getGroupsSize(); i++) {
				for (int j = 0; j < v.getStorage().getChannelsSize(i); j++) {
					if (!v.getStorage().isXChannel(
							v.getStorage().getChannel(i, j))) {
						LegendTreeRenderer r = new LegendTreeRenderer(v);
						r.setPrintMode(true);
						JPanel p = (JPanel) r.getTreeCellRendererComponent(
								tree, v.getStorage().getChannelNode(i, j),
								false, false, true, 1, false);
						legendPanel.add(p);
					}
				}
			}

			legendPanel.add(Box.createVerticalGlue());
	
			if (options.isPrintLegendBorder()) {
				javax.swing.border.Border selectedBorder = javax.swing.BorderFactory
						.createLineBorder(UIManager
								.getColor("Tree.selectionBorderColor"), 1);
				legendPanel.setBorder(selectedBorder);
			}

			return legendPanel;
		}

		/**
		 * Returns print frame
		 * 
		 * @return print frame
		 */
		public JFrame getPrintFrame() {
			return printFrame;
		}

		/**
		 * Returns print preview panel
		 * 
		 * @return print preview panel
		 */
		public JComponent getPreviewPanel() {
			GraphicsConfiguration gc = printFrame.getGraphicsConfiguration();

			Image printImage = gc.createCompatibleImage(printFrame
					.getContentPane().getWidth(), printFrame.getContentPane()
					.getHeight());
			Graphics2D g2 = (Graphics2D) printImage.getGraphics();
			printFrame.getContentPane().print(g2);
			g2.dispose();

			BufferedImage bi = new BufferedImage(printImage.getWidth(null),
					printImage.getHeight(null), BufferedImage.TYPE_INT_RGB);
			g2 = bi.createGraphics();
			g2.drawImage(printImage, null, null);
			g2.dispose();

			AffineTransformOp op = new AffineTransformOp(AffineTransform
					.getScaleInstance(zoom, zoom),
					AffineTransformOp.TYPE_BILINEAR);

			ImageIcon icon = new ImageIcon(op.filter(bi, null));
			previewLabel.setIcon(icon);
			previewLabel.setPreferredSize(this.getPreferredSize());

			return this;
		}

		/**
		 * Sets zoom factor for this component
		 * 
		 * @param zoom
		 *            zoom factor (1.0 = 100%)
		 */
		public void setZoom(double zoom) {
			this.zoom = zoom;
		}

		/**
		 * Components preferred size
		 * 
		 * @return preferred size
		 */
		public Dimension getPreferredSize() {
			return new Dimension((int) Math.abs(format.getWidth() * zoom),
					(int) Math.abs(format.getHeight() * zoom));
		}

		/**
		 * Components maximum size
		 * 
		 * @return maximum size
		 */
		public Dimension getMaximumSize() {
			return getPreferredSize();
		}

		/**
		 * Components minimum size
		 * 
		 * @return minimum size
		 */
		public Dimension getMinimumSize() {
			return getPreferredSize();
		}

		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() == 2) {
				options.setVisible(true);
			}

		}

		public void mouseEntered(MouseEvent e) {
			// TODO Auto-generated method stub

		}

		public void mouseExited(MouseEvent e) {
			// TODO Auto-generated method stub

		}

		public void mousePressed(MouseEvent e) {
			// TODO Auto-generated method stub

		}

		public void mouseReleased(MouseEvent e) {
			// TODO Auto-generated method stub

		}

	}

}