/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.Box;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;

import com.simulogic.dataview.legend.LegendTree;

/**
 * Class that contains different data views
 */
public class ViewContent extends JPanel {

	private JTabbedPane tabs;

	private DataStorage storage;

	private LegendTree legend;

	private DataView view;

	private javax.swing.JTextField comment;

	/**
	 * Creates new ViewContent
	 */
	public ViewContent() {
		super();
		this.setLayout(new java.awt.GridLayout());
		storage = new DataStorage();
		createLegend();
	}

	/**
	 * Returns Plot/View tabbed pane
	 * 
	 * @return tabbed pane
	 */
	public JTabbedPane getTabs() {
		return tabs;
	}

	/**
	 * Sets current DataStorage
	 * 
	 * @param s
	 *            new DataStorage
	 */
	public void setStorage(DataStorage s) {
		storage = s;
	}

	/**
	 * Returns current DataStorage
	 * 
	 * @return DataStorage
	 */
	public DataStorage getStorage() {
		return storage;
	}

	/**
	 * Returns current LegendTree
	 * 
	 * @return LegendTree
	 */
	public LegendTree getLegend() {
		return legend;
	}

	/**
	 * Returns current DataView
	 * 
	 * @return DataView
	 */
	public DataView getView() {
		return view;
	}

	/**
	 * Returns comment
	 * 
	 * @return comment
	 */
	public String getComment() {
		return comment.getText();
	}

	private JScrollPane createDataView() {
		view = new DataView(storage);
		return view.getScrollPane();
	}

	private JScrollPane createDataTable() {
		JTable table = new javax.swing.JTable(new DataTableModel(storage));
		return createTableScrollPane(table);
	}

	private JScrollPane createCursorsTable() {
		JTable table = new javax.swing.JTable(new DataTableModel(view
				.getCursors().getCursorsStorage()));
		return createTableScrollPane(table);
	}

	private JScrollPane createTableScrollPane(JTable table) {
		JScrollPane tableScrl = new JScrollPane(table);
		tableScrl
				.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		table.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
		RowHeaderRenderer rowHeader = new RowHeaderRenderer();
		table.setDefaultRenderer(Integer.class, rowHeader);
		table.setDefaultEditor(Double.class, new DoubleEditor());
		table.setCellSelectionEnabled(true);
		table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

		addMouseListenerToHeaderInTable(table);

		return tableScrl;
	}

	private void addMouseListenerToHeaderInTable(JTable table) {
		final JTable tableView = table;
		MouseAdapter listMouseListener = new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				tableView.clearSelection();
				TableColumnModel columnModel = tableView.getColumnModel();
				int viewColumn = columnModel.getColumnIndexAtX(e.getX());
				int column = tableView.convertColumnIndexToModel(viewColumn);
				if (e.getClickCount() == 1 && column != -1) {
					tableView.setColumnSelectionInterval(column, column);
					tableView.addRowSelectionInterval(0, tableView
							.getRowCount() - 1);
				}
			}
		};
		JTableHeader th = tableView.getTableHeader();
		th.addMouseListener(listMouseListener);
	}

	private void createLegend() {
		tabs = new javax.swing.JTabbedPane();
		tabs.setTabPlacement(javax.swing.SwingConstants.BOTTOM);
		tabs.addTab(ResourceManager.getResource("Plot"), createDataView());
		tabs.addTab(ResourceManager.getResource("Data"), createDataTable());
		tabs.addTab(ResourceManager.getResource("Cursors"),
				createCursorsTable());

		legend = new LegendTree(view);
		JPanel legendPanel = new JPanel(new java.awt.BorderLayout());
		JScrollPane legendScrl = new JScrollPane(legend.getTreeComponent());
		legendPanel.add(legendScrl, "Center");
		Box commentPanel = Box.createHorizontalBox();
		commentPanel.add(Box.createHorizontalStrut(5));
		commentPanel.add(new javax.swing.JLabel(
				com.simulogic.dataview.ResourceManager.getResource("Comment_")));
		commentPanel.add(Box.createHorizontalStrut(5));
		comment = new javax.swing.JTextField(20);
		commentPanel.add(comment);
		legendPanel.add(commentPanel, "South");

		javax.swing.JSplitPane vSplitPane = new javax.swing.JSplitPane(
				javax.swing.SwingConstants.VERTICAL);
		vSplitPane.setOneTouchExpandable(true);
		vSplitPane.setRightComponent(tabs);
		vSplitPane.setLeftComponent(legendPanel);
		vSplitPane.setDividerLocation(0.0D);
		this.add(vSplitPane);
	}

}
